/*
 * (c) Copyright 2017 Palantir Technologies Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.palantir.conjure.java.client.retrofit2;

import com.google.common.util.concurrent.ListenableFuture;
import java.time.LocalDate;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import okhttp3.ResponseBody;
import retrofit2.Call;
import retrofit2.http.Body;
import retrofit2.http.GET;
import retrofit2.http.Header;
import retrofit2.http.Headers;
import retrofit2.http.POST;
import retrofit2.http.Path;
import retrofit2.http.Query;

public interface TestService {
    @GET(".")
    Call<String> get();

    @GET("getGuavaOptionalString/{pathString}/")
    Call<com.google.common.base.Optional<String>> getGuavaOptionalString(
            @Path("pathString") com.google.common.base.Optional<String> pathString,
            @Query("queryString") com.google.common.base.Optional<String> queryString);

    @GET("getJava8OptionalString/{pathString}/")
    Call<Optional<String>> getJava8OptionalString(
            @Path("pathString") Optional<String> pathString, @Query("queryString") Optional<String> queryString);

    @GET("getJava8OptionalInt")
    Call<java.util.OptionalInt> getJava8OptionalInt();

    @GET("getJava8OptionalLong")
    Call<java.util.OptionalLong> getJava8OptionalLong();

    @GET("getJava8OptionalDouble")
    Call<java.util.OptionalDouble> getJava8OptionalDouble();

    @POST("getComplexGuavaType")
    Call<com.google.common.base.Optional<LocalDate>> getComplexGuavaType(
            @Body com.google.common.base.Optional<LocalDate> date);

    @POST("getComplexJavaType")
    Call<Optional<LocalDate>> getComplexJava8Type(@Body Optional<LocalDate> date);

    @GET("relative")
    Call<String> getRelative();

    @GET("getComplexCborType")
    @Headers("Accepts: application/cbor")
    Call<Optional<LocalDate>> getComplexCborType();

    @POST("makeCborRequest")
    @Headers("Content-Type: application/cbor")
    Call<Void> makeCborRequest(@Body LocalDate localDate);

    @POST("makeCompletableFutureRequest")
    @Headers("hr-path-template: /makeCompletableFutureRequest")
    CompletableFuture<String> makeCompletableFutureRequest();

    @POST("makeListenableFutureRequest")
    @Headers("hr-path-template: /makeListenableFutureRequest") // conjure-java adds these headers
    ListenableFuture<String> makeListenableFutureRequest();

    @GET("getJavaOptionalHeader")
    ListenableFuture<String> getJavaOptionalHeader(@Header("Optional-Header") Optional<String> optionalHeader);

    @GET("getJavaOptionalHeader")
    ListenableFuture<String> getGuavaOptionalHeader(
            @Header("Optional-Header") com.google.common.base.Optional<String> optionalHeader);

    @GET("getResponseBody")
    ListenableFuture<ResponseBody> getResponseBody();

    /** This is to ensure that conjure-java 4.x clients still work. */
    @GET("getCallOfResponseBody")
    Call<ResponseBody> getCallOfResponseBody();
}
